#pragma once

#include <iostream>
#include <fstream>
#include <algorithm>
#include <iterator>
#include <vector>
#include <deque>
#include <list>
#include <string>
#include <sstream>


void createData_stream(size_t num) {
	std::ofstream fout("data.txt");
	for (int i = 0; i < num-1; ++i)
		fout << -50 + rand() % 101 << ' ';
	fout << -50 + rand() % 101;
	fout << std::endl;
}

void createData_generate(size_t num) {
	std::vector<int> myvec(num);
	std::generate(myvec.begin(), myvec.end(), []() { return -50 + rand() % 101; });

	std::ofstream fout("data.txt", std::ios::app);
	std::ostream_iterator<int> fout_it(fout, " ");
	std::copy(myvec.begin(), myvec.end() - 1, fout_it);
	fout << *(myvec.end() - 1) << std::endl;
}


template<typename T>
T loadData_stream(const std::string& filename) {
	T container;

	std::ifstream fin(filename);
	if (!fin) {
		std::cerr << "loadData_stream error: cannot open file" << std::endl;
		exit;
	}

	std::string line;
	bool no_errors(true);
	while (std::getline(fin, line, ' ')&&no_errors) {
		std::istringstream iss(line);
		typename T::value_type elem;
		if (!(iss >> elem))
			no_errors = false;
		else
			container.push_back(elem);
	}
	return container;
}

template<typename T>
T loadData_copy(const std::string& filename, size_t num) {

	T container(num);
	std::ifstream fin(filename);
	if (!fin) {
		std::cerr << "loadData_copy error: cannot open file" << std::endl;
		exit;
	}else {
		std::copy(std::istream_iterator<typename T::value_type>(fin), std::istream_iterator<typename T::value_type>(), container.begin()); 
		return container;
	}
}


template<typename iter>
typename iter::value_type first_odd(iter begin, iter end)
{
	return *std::find_if(begin, end, [](const auto& elem) { return std::abs(elem) % 2 == 1; });
}

template<typename T>
T modify_template(const T& container)
{
	T res;
	auto odd = first_odd(container.begin(), container.end());
	
	for (const auto& elem : container)
		res.push_back(elem+odd);
	return res;
}

template<typename input_iter, typename out_iter>
void modify_iter(const input_iter in_beg, const input_iter in_end, out_iter out_beg) {
	auto iter = out_beg;
	auto odd = first_odd(in_beg, in_end);
	for (auto it = in_beg; it != in_end; ++it)
		*(iter++) = *it + odd;
}

template<typename T>
void modify_for_each(T& container) {
	auto odd = first_odd(container.begin(), container.end());
	std::for_each(container.begin(), container.end(), [&odd](const auto& elem) { return elem + odd; });
}

template<typename T>
void modify_transform(const std::string& input_fname, const std::string& output_fname) {
	std::ifstream fin(input_fname);
	std::ofstream fout(output_fname, std::ios::app);

	if (!fin)
		std::cerr << "modify_tranform error: cannot open file" << std::endl;
	else {
	    
		auto odd = first_odd(std::istream_iterator<T>(fin), std::istream_iterator<T>());
		fin.seekg(0, std::ios::beg);
		
		std::transform(std::istream_iterator<T>(fin), std::istream_iterator<T>(), std::ostream_iterator<T>(fout, " "), [odd](const auto& elem) { return elem + odd; });
		
		
		fout << std::endl;
	};

}


template <typename T>
void outputResult_stream(const std::string& filename, const T& cont) {
	std::ofstream fout(filename, std::ios::app);
	for (const auto& elem : cont)
		fout << elem << ' ';
	fout << std::endl;
}

template <typename T>
void outputResult_copy(const std::string& filename, const T& cont) {
	std::ofstream fout(filename, std::ios::app);
	std::copy(cont.begin(), cont.end(), std::ostream_iterator<typename T::value_type>(fout, ' '));
	fout << std::endl;
}

