#include "Container.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <ranges>
#include <algorithm>

subscriber::subscriber(const char* surname,
	                   const char* region,
	                   const char* address,
	                   const char* phone_number,
				       size_t      ontract_number,
	                   const char* contract_date,
				       double      setup_price,
				       double      monthly_price,
				       const char* last_payment_date):
				       contract_number(ontract_number),
				       setup_price    (setup_price),
				       monthly_price  (monthly_price)
{
	strcpy_s(this->surname, 30, surname);
	strcpy_s(this->region, 30, region);
	strcpy_s(this->address, 30, address);
	strcpy_s(this->phone_number, 15, phone_number);
	strcpy_s(this->contract_date, 15, contract_date);
	strcpy_s(this->last_payment_date, 15, last_payment_date);
};

std::istream& operator>>(std::istream& in, subscriber& subscriber)
{
	std::cout << "Surname:" << std::endl;
	in.getline(subscriber.surname, 30);
	std::cout << "Region:" << std::endl;
	in.getline(subscriber.region, 30);
	std::cout << "Address:" << std::endl;
	in.getline(subscriber.address, 30);
	std::cout << "Phone number:" << std::endl;
	in.getline(subscriber.phone_number, 15);
	std::cout << "Contract number:" << std::endl;
	in >> subscriber.contract_number;
	in.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	std::cout << "Contract date:" << std::endl;
	in.getline(subscriber.contract_date, 15);
	std::cout << "Setup price:" << std::endl;
	in >> subscriber.setup_price;
	std::cout << "Monthly price:" << std::endl;
	in >> subscriber.monthly_price;
	in.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
	std::cout << "Last payment date:" << std::endl;
	in.getline(subscriber.last_payment_date, 15);
	return in;
}

std::ostream& operator<<(std::ostream& out, const subscriber& subscriber)
{
	out << subscriber.surname			<< std::endl
		<< subscriber.region			<< std::endl
		<< subscriber.address			<< std::endl
		<< subscriber.phone_number		<< std::endl
		<< subscriber.contract_number	<< std::endl
		<< subscriber.contract_date		<< std::endl
		<< subscriber.setup_price		<< std::endl
		<< subscriber.monthly_price		<< std::endl
		<< subscriber.last_payment_date << std::endl;
	return out;
}

void Container::sync()
{
	std::ifstream fin(filename, std::ios::binary);
	subscriber elem;
	size_t i = 0;
	while (fin.read(reinterpret_cast<char*>(&elem), sizeof(subscriber))) {
		i++;
		array.insert(std::make_pair(i, elem));
	}

}

Container::Container():
	filename("data"),
	edits(0)
{}

Container::Container(const std::string& filename):
	filename(filename),
	edits(0)
{}

Container::Container(const std::string& filename, const std::initializer_list<subscriber>& list):
	filename(filename),
	edits(0)
{
	write(list);
}

void Container::write(const std::initializer_list<subscriber>& list)
{
	std::ofstream fout(filename, std::ios::binary | std::ios::app);
	for (const auto& elem : list)
		fout.write(reinterpret_cast<const char*>(&elem), sizeof(subscriber));

}

void Container::open(bool fill)
{
	std::ofstream fout(filename, std::ios::binary);
	if (!fout) {
		std::cerr << "open error: unable to create file.";
		exit;
	}

	if (fill) {
		subscriber elem;
		bool continue_filling;
		do {
			std::cin >> elem;
			std::cout << "Input:" << std::endl
					  << "0 - to stop filling;" << std::endl
					  << "1 - to continue;" << std::endl;
			fout.write(reinterpret_cast<char*>(&elem), sizeof(subscriber));
			std::cin >> continue_filling;
			std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
		} while (continue_filling);
	}
}

void Container::remove(size_t index)
{
	sync();
	array.erase(index);
	std::ofstream fout(filename + "_edit_" + std::to_string(++edits), std::ios::binary);
	for (const auto& elem : array)
		fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(subscriber));
}

void Container::replace(size_t index, const subscriber& sub)
{
	sync();
	array[index] = sub;
	std::ofstream fout(filename + "_edit_" + std::to_string(++edits), std::ios::binary);
	for (const auto& elem : array)
		fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(subscriber));
}

void Container::to_text()
{
	sync();
	std::ofstream fout(filename + ".txt");
	
		fout << "|"<<std::setw(3)<<"#"<< "|" 
			 << std::setw(15) << "Surname" <<"|"
			 << std::setw(15) << "Region" << "|" 
			 << std::setw(15) << "Address" << "|"
			 << std::setw(15) << "Phone Number" << "|"
			 << std::setw(15) << "Contract Number" << "|"
			 << std::setw(15) << "Contract Date"  << "|"
			 << std::setw(15) << "Setup Price"  << "|" 
			 << std::setw(15) << "Monthly Price" << "|"
			 << std::setw(15) << "Last Payment" << "|" << std::endl;
	
	for (const auto& elem : array) {
		fout << "|" << std::setw(3) << elem.first << "|"
			<< std::setw(15) << elem.second.surname << "|"
			<< std::setw(15) << elem.second.region << "|"
			<< std::setw(15) << elem.second.address << "|"
			<< std::setw(15) << elem.second.phone_number << "|"
			<< std::setw(15) << elem.second.contract_number << "|"
			<< std::setw(15) << elem.second.contract_date << "|"
			<< std::setw(15) << elem.second.setup_price << "|"
			<< std::setw(15) << elem.second.monthly_price << "|"
			<< std::setw(15) << elem.second.last_payment_date << "|"
			<< std::endl;
	}
}

const std::string& Container::get_filename()
{
	return filename;
}

void Container::search_by_region(const std::string& reg)
{
	sync();
	std::ofstream fout(filename+"_reg_"+reg, std::ios::binary);

	std::vector<subscriber> res;
	
	auto iter = array.begin();
	while (iter!=array.end())
	{
		iter = std::find_if(iter, array.end(), [&reg](const auto& elem) { return elem.second.region == reg; });
		if (iter != array.end()) {
			res.push_back(iter->second);
			++iter;
		}
	}

	for (const auto& elem : res)
		fout.write(reinterpret_cast<const char*>(&elem), sizeof(subscriber));
}

void Container::search_by_contract_date(const std::string& date)
{
	sync();
	std::ofstream fout(filename + "_cd_" + date, std::ios::binary);

	std::vector<subscriber> res;

	auto iter = array.begin();
	while (iter != array.end())
	{
		iter = std::find_if(iter, array.end(), [&date](const auto& elem) { return elem.second.contract_date == date; });
		if (iter != array.end()) {
			res.push_back(iter->second);
			++iter;
		}
	}

	for (const auto& elem : res)
		fout.write(reinterpret_cast<const char*>(&elem), sizeof(subscriber));
}

void Container::search_by_payment_date(const std::string& date)
{
	sync();
	std::ofstream fout(filename + "_pd_" + date, std::ios::binary);

	std::vector<subscriber> res;

	auto iter = array.begin();
	while (iter != array.end())
	{
		iter = std::find_if(iter, array.end(), [&date](const auto& elem) { return elem.second.last_payment_date == date; });
		if (iter != array.end()) {
			res.push_back(iter->second);
			++iter;
		}
	}

	for (const auto& elem : res)
		fout.write(reinterpret_cast<const char*>(&elem), sizeof(subscriber));
}
